/******************************************************************************

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#include "errorwriter.hpp"

ErrorWriter::ErrorWriter(Wordlist &wl, Settings &se)
     : wordlist(wl)
     , settings(se)
     , filename(settings.getString("errorWriter/filename", "out.xml"))
     , reportDamerau(settings.getString("damerau/reportName", "damerau"))
     , reportSplitCompound(settings.getString("splitCompound/reportName", "split"))
     , reportSoundError(settings.getString("soundError/reportName", "sound"))
     , reportErrorIntroducer(settings.getString("errorIntroducer/reportName", "introduced"))
     {
     impl = DOMImplementationRegistry::getDOMImplementation(XMLtr("Core"));
     document = impl->createDocument(0, XMLtr("errors"), 0);
     root = document->getDocumentElement();

     document->setStandalone(true);
}

ErrorWriter::~ErrorWriter() {
     delete impl;
}

void ErrorWriter::write() const {
     std::ofstream out(filename.c_str());
     if (!out) {
	  std::cerr << "Error: Could not open file '" << filename << "'\n";
	  return;
     }
     MyWriterFilter filter(settings);

     DOMWriter *writer = impl->createDOMWriter();
     writer->setFilter(&filter);

     if (writer->canSetFeature(XMLtr("format-pretty-print"), true)) {
	  writer->setFeature(XMLtr("format-pretty-print"), true);
     }

     // add final newline to put end the tag of the root element on separate line
     DOMText *newline = document->createTextNode(XMLtr("\n"));
     root->appendChild(newline);

     out << XMLtr(writer->writeToString(*document));
}

void ErrorWriter::insertInfo(char *argv[], const std::string& seed, const std::string& file) {
     try {
	  DOMElement *errorElem = addHeader("info");

	  addElement(errorElem, "seed", seed);

	  std::string parameters;
	  for (int i = 1; argv[i]; ++i) {
	       parameters += std::string(" ") + argv[i];
	  }

	  addElement(errorElem, "settingsFile", file);
	  addElement(errorElem, "parameters", parameters);

	  addFooter(errorElem);
     }
     catch (...) {
	  std::cerr << "Error: Unexpected exception. ew1\n";
     }     
}

void ErrorWriter::insertDamerau(const std::string& reason, const int errorIndex, const WordTag& wtold, const WordTag& wtnew) {
     try {
	  DOMElement *errorElem = addHeader(reportDamerau);

	  addElement(errorElem, "type", reason);
	  addElement(errorElem, "origIndex", boost::lexical_cast<std::string>(wtold.orig()));
	  addElement(errorElem, "fromIndex", boost::lexical_cast<std::string>(wtold.row()));
	  addElement(errorElem, "toIndex", boost::lexical_cast<std::string>(wtold.row()));
	  addElement(errorElem, "errorIndex", boost::lexical_cast<std::string>(errorIndex));
	  addElement(errorElem, "oldWord", wtold.word());
	  addElement(errorElem, "oldTag", wordlist.tag(wtold.tag()));
	  addElement(errorElem, "newWord", wtnew.word());
	  addElement(errorElem, "newTag", wordlist.tag(wtnew.tag()));

	  addFooter(errorElem);
     }
     catch (boost::bad_lexical_cast &) {
	  std::cerr << "Could not cast int to string.\n";
     }
     catch (...) {
	  std::cerr << "Error: Unexpected exception. ew2\n";
     }
}

void ErrorWriter::insertSplitCompound(const int errorIndex, const WordTag& wtold, const WordTag& wtnew1, const WordTag& wtnew2) {
     try {
	  DOMElement *errorElem = addHeader(reportSplitCompound);

	  addElement(errorElem, "origIndex", boost::lexical_cast<std::string>(wtold.orig()));
	  addElement(errorElem, "fromIndex", boost::lexical_cast<std::string>(wtold.row()));
	  addElement(errorElem, "toIndex", boost::lexical_cast<std::string>(wtold.row()));
	  addElement(errorElem, "errorIndex", boost::lexical_cast<std::string>(errorIndex));
	  addElement(errorElem, "oldWord", wtold.word());
	  addElement(errorElem, "oldTag", wordlist.tag(wtold.tag()));
	  addElement(errorElem, "newWord1", wtnew1.word());
	  addElement(errorElem, "newTag1", wordlist.tag(wtnew1.tag()));
	  addElement(errorElem, "newWord2", wtnew2.word());
	  addElement(errorElem, "newTag2", wordlist.tag(wtnew2.tag()));

	  addFooter(errorElem);
     }
     catch (boost::bad_lexical_cast &) {
	  std::cerr << "Could not cast int to string.\n";
     }
     catch (...) {
	  std::cerr << "Error: Unexpected exception. ew3\n";
     }
}

void ErrorWriter::insertIntroduced(const std::string& desc, const WordTag& wtold, const WordTag& wtnew) {
     try {
	  DOMElement *errorElem = addHeader(reportErrorIntroducer);

	  addElement(errorElem, "type", desc);
	  addElement(errorElem, "origIndex", boost::lexical_cast<std::string>(wtold.orig()));
	  addElement(errorElem, "fromIndex", boost::lexical_cast<std::string>(wtold.row()));
	  addElement(errorElem, "toIndex", boost::lexical_cast<std::string>(wtold.row()));
	  addElement(errorElem, "oldWord", wtold.word());
	  addElement(errorElem, "oldTag", wordlist.tag(wtold.tag()));
	  addElement(errorElem, "newWord", wtnew.word());
	  addElement(errorElem, "newTag", wordlist.tag(wtnew.tag()));

	  addFooter(errorElem);
     }
     catch (boost::bad_lexical_cast &) {
	  std::cerr << "Could not cast int to string.\n";
     }
     catch (...) {
	  std::cerr << "Error: Unexpected exception. ew4\n";
     }
}

void ErrorWriter::insertIntroduced(const std::string& desc, int origIndex, int fromIndex, int toIndex, const std::string& oldWord, int oldTag, const std::string& newWord, int newTag) {
     try {
	  DOMElement *errorElem = addHeader(reportErrorIntroducer);

	  addElement(errorElem, "type", desc);
	  addElement(errorElem, "origIndex", boost::lexical_cast<std::string>(origIndex));
	  addElement(errorElem, "fromIndex", boost::lexical_cast<std::string>(fromIndex));
	  addElement(errorElem, "toIndex", boost::lexical_cast<std::string>(toIndex));
	  addElement(errorElem, "oldWord", oldWord);
	  addElement(errorElem, "oldTag", wordlist.tag(oldTag));
	  addElement(errorElem, "newWord", newWord);
	  addElement(errorElem, "newTag", wordlist.tag(newTag));

	  addFooter(errorElem);
     }
     catch (boost::bad_lexical_cast &) {
	  std::cerr << "Could not cast int to string.\n";
     }
     catch (...) {
	  std::cerr << "Error: Unexpected exception. ew5\n";
     }
}

void ErrorWriter::insertSoundError(const WordTag& wtold, const WordTag& wtnew) {
     try {
	  DOMElement *errorElem = addHeader(reportSoundError);

	  addElement(errorElem, "origIndex", boost::lexical_cast<std::string>(wtold.orig()));
	  addElement(errorElem, "fromIndex", boost::lexical_cast<std::string>(wtold.row()));
	  addElement(errorElem, "toIndex", boost::lexical_cast<std::string>(wtold.row()));
	  addElement(errorElem, "oldWord", wtold.word());
	  addElement(errorElem, "oldTag", wordlist.tag(wtold.tag()));
	  addElement(errorElem, "newWord", wtnew.word());
	  addElement(errorElem, "newTag", wordlist.tag(wtnew.tag()));

	  addFooter(errorElem);
     }
     catch (boost::bad_lexical_cast &) {
	  std::cerr << "Could not cast int to string.\n";
     }
     catch (...) {
	  std::cerr << "Error: Unexpected exception. ew6\n";
     }
}

DOMElement* ErrorWriter::addHeader(const std::string header) {
     root->appendChild(document->createTextNode(XMLtr("\n  ")));
     DOMElement *errorElem = document->createElement(XMLtr(header.c_str()));
     root->appendChild(errorElem);
     return errorElem;
}

void ErrorWriter::addFooter(DOMElement *elem) {
     elem->appendChild(document->createTextNode(XMLtr("\n  ")));
}

void ErrorWriter::addElement(DOMElement *errorElem, const std::string tag, const std::string& value) {
     DOMText *newline = document->createTextNode(XMLtr("\n    "));
     errorElem->appendChild(newline);
     DOMElement *elem = document->createElement(XMLtr(tag.c_str()));
     errorElem->appendChild(elem);
     DOMText *text = document->createTextNode(XMLtr(value.c_str()));
     elem->appendChild(text);
}
