/******************************************************************************

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#include "settings.hpp"

Settings::Settings(const std::string& filename) {
    parser = new XercesDOMParser();
    errHandler = new MyErrorHandler();
    
    parser->setCreateCommentNodes(false);
    parser->setDoValidation(true);
    parser->setDoNamespaces(true);
    parser->setDoSchema(true);
    parser->setIncludeIgnorableWhitespace(false);
    parser->setValidationScheme(XercesDOMParser::Val_Always);
    parser->setErrorHandler(errHandler);
    try {
        parser->parse(filename.c_str());
    }
    catch (const XMLException& toCatch) {
        char* message = XMLtr(toCatch.getMessage());
        std::cerr << "Exception message is: \n"
                  << message << "\n";
        delete [] message;
        XMLPlatformUtils::Terminate();
        std::exit(1);
    }
    catch (const DOMException& toCatch) {
        std::cerr << "Exception message is: \n"
                  << toCatch.code << "\n";
        XMLPlatformUtils::Terminate();
        std::exit(1);
    }
    catch (...) {
        std::cerr << "Unexpected XML parse exception while parsing settings file (please check if the file exists)\n";
        XMLPlatformUtils::Terminate();
        std::exit(1);
    }
     
    DOMElement *root = parser->getDocument()->getDocumentElement();
    if (XMLcmp(root->getTagName(), XMLtr("settings")) != 0) {
        std::cerr << "This is not a settings file!\n";
    }    
}
     
Settings::~Settings() {
    std::cerr << "~parser";
    delete parser;
    std::cerr << "~errHandler";
    delete errHandler;
}

Settings::SubParameters Settings::allSubParameters(const std::string&, const std::list<std::string>&) const {
    throw NotFoundException("Not implemented yet!");
}

int Settings::toInt(const std::string& value, int def) {
    try {
        return boost::lexical_cast<int>(value);
    }
    catch (boost::bad_lexical_cast &) {
        std::cerr << "Warning: '" << value << "' is not an integer value.\n";
    }
    catch (...) {
        std::cerr << "Error: Unexpected exception se1\n";
    }
    return def;
}

int Settings::getInt(const std::string& path, int def) const {
    try {
        return toInt(parameterValue(path), def);
    }
    catch (ParamNotFoundException &) {
        std::cerr << "Warning: Could not read setting '" << path << "'.\n";
    }
    return def;
}

std::string Settings::getString(const std::string& path, const std::string& def) const {
    try {
        return parameterValue(path);
    }
    catch (ParamNotFoundException &) {
        std::cerr << "Warning: Could not read setting '" << path << "'.\n";
    }
    catch (...) {
        std::cerr << "Error: Unexpected exception se2\n";
    }
    return def;
}

bool Settings::toBool(const std::string& value, bool def) {
    try {
        boost::regex exp;
        exp.set_expression("1|yes|true", boost::regbase::normal|boost::regbase::icase);
        if (regex_match(value, exp)) {
            return true;
        }
        exp.set_expression("0|no|false", boost::regbase::normal|boost::regbase::icase);
        if (regex_match(value, exp)) {
            return false;
        }
        throw NotBooleanValueException();
    }
    catch (NotBooleanValueException &e) {
        std::cerr << "Warning: Not boolean value '" << value << "'.\n";
    }
    catch (...) {
        std::cerr << "Error: Unexpected exception se3\n";
    }
    return def;
}

bool Settings::getBool(const std::string& path, bool def) const {
    std::string ret;
    try {
        ret = parameterValue(path);
    }
    catch (ParamNotFoundException &) {
        std::cerr << "Warning: Could not read setting '" << path << "'.\n";
        return def;
    }
    return toBool(ret, def);
}

char Settings::toChar(const std::string& value, char def) {
    try {
        if (value.length() != 1) {
            throw NotCharException();
        }
        return value[0];
    }
    catch (NotCharException &) {
        std::cerr << "Warning: Not a char value '" << value << "'.\n";
    }
    catch (...) {
        std::cerr << "Error: Unexpected exception se4\n";
    }
    return def;
}

char Settings::getChar(const std::string& path, char def) const {
    try {
        return toChar(parameterValue(path), def);
    }
    catch (ParamNotFoundException &) {
        std::cerr << "Warning: Could not read setting '" << path << "'.\n";
    }
    return def;
}

double Settings::toDouble(const std::string& value, double def) {
    try {
        return boost::lexical_cast<double>(value);
    }
    catch (boost::bad_lexical_cast &) {
        std::cerr << "Warning: '" << value << "' is not a double value.\n";
    }
    catch (...) {
        std::cerr << "Error: Unexpected exception se5\n";
    }
    return def;
}

double Settings::getDouble(const std::string& path, double def) const {
    try {
        return boost::lexical_cast<double>(parameterValue(path));
    }
    catch (boost::bad_lexical_cast &) {
        std::cerr << "Warning: Setting '" << path << "' not a double value.\n";
    }
    catch (ParamNotFoundException &) {
        std::cerr << "Warning: Could not read setting '" << path << "'.\n";
    }
    catch (...) {
        std::cerr << "Error: Unexpected exception se6\n";
    }
    return def;
}

