/******************************************************************************

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

******************************************************************************/

#include "splitcompound.hpp"

SplitCompound::SplitCompound(Wordlist &wl, Settings &se, ErrorWriter &ew, InputReader &ir) 
     : wordlist(wl), settings(se), errorwriter(ew), inputreader(ir) {
     splitUnknownWords = settings.getBool("splitCompound/splitUnknownWords", true);
     splitThreshold = settings.getInt("splitCompound/splitThreshold", 20);
     minWordLength = settings.getInt("splitCompound/minWordLength", 4);
     minSplitWordLength = settings.getInt("splitCompound/minSplitWordLength", 2);
     wordLengthFactor = settings.getInt("splitCompound/factors/wordLength", 1);
     inDictionaryFirstFactor = settings.getInt("splitCompound/factors/inDictionaryFirst", 10);
     inDictionarySecondFactor = settings.getInt("splitCompound/factors/inDictionarySecond", 10);
     tagAllowedFactor = settings.getInt("splitCompound/factors/tagAllowed", 10);
     tagMatchFirstFactor = settings.getInt("splitCompound/factors/tagMatchFirst", 0);
     tagMatchSecondFactor = settings.getInt("splitCompound/factors/tagMatchSecond", 10);
     allowedClasses = settings.allParameters("splitCompound/allowedClasses", "class");
     reportSplitCompound = settings.getString("splitCompound/reportName", "split");
}

unsigned int SplitCompound::applyHelper(const std::string& word, const int tag,
					const unsigned int row,
					const std::string& origWord, const int origTag,
					const unsigned int orig,
					const int inDictionaryFactor,
					const int tagMatchFactor,
					WordTag &max) const {
     unsigned int currentValue = 0;
     WordTag current(word, wordlist.getTag(word, tag), row, origWord, origTag, orig);
     Wordlist::const_iterator iter_last = wordlist.findLastWord(word);
     for (Wordlist::const_iterator iter = wordlist.findFirstWord(word);
	  iter != iter_last; ++iter) {
	  unsigned int tempValue = inDictionaryFactor;
	  if (iter->tag() == tag) {
	       tempValue += tagAllowedFactor + tagMatchFactor;
	  }
	  else if (tagAllowed(iter->tag())) {
	       tempValue += tagAllowedFactor;
	  }
	  if (tempValue > currentValue) {
	       currentValue = tempValue;
	       current.set(iter->word(), iter->tag());
	  }
     }
     max = current;
     return currentValue;
}

std::pair<bool, std::pair<WordTag, WordTag> > SplitCompound::apply(const WordTag& wt) const {
     WordTag maxFirst;
     WordTag maxSecond;
     unsigned int maxFactorValueFirst = 0;
     unsigned int maxFactorValueSecond = 0;
     if ((splitUnknownWords || wordlist.lookup(wt.word())) 
	 && wt.word().length() >= minWordLength) {
	  for (unsigned int i = minSplitWordLength; i < wt.word().length() - minSplitWordLength + 1; i++) {
	       // FIXME stt wordtag->desc
	       WordTag currentFirst;
	       WordTag currentSecond;
	       unsigned int firstVal = applyHelper(wt.word().substr(0, i), wt.tag(), wt.row(), wt.origWord(), wt.origTag(), wt.orig(), inDictionaryFirstFactor, tagMatchFirstFactor, currentFirst);
	       unsigned int secondVal = applyHelper(wt.word().substr(i), wt.tag(), wt.row(), wt.origWord(), wt.origTag(), wt.orig(), inDictionarySecondFactor, tagMatchSecondFactor, currentSecond);
	       if (firstVal + secondVal > maxFactorValueFirst + maxFactorValueSecond) {
		    maxFactorValueFirst = firstVal;
		    maxFactorValueSecond = secondVal;
		    maxFirst = currentFirst;
		    maxSecond = currentSecond;
	       }
	  }
     }
     unsigned int maxFactorValue = maxFactorValueFirst + maxFactorValueSecond + wt.word().length() * wordLengthFactor;
     //std::cerr << maxFirst.word() << " " << maxSecond.word() << " " << maxFactorValue << "(" << splitThreshold << ")\n";
     if (maxFactorValue < splitThreshold) {
	  return std::make_pair(false, std::make_pair(wt, wt));
     }
     return std::make_pair(true, std::make_pair(maxFirst, maxSecond));
}

bool SplitCompound::apply(std::list<WordTag>& listwt) const {
     if (listwt.front().blank()) {
	  return false;
     }
     std::pair<bool, std::pair<WordTag, WordTag> > ret = apply(listwt.front());
     if (ret.first) {
	  ret.second.second.setRow(ret.second.second.row() + 1);

	  ret.second.first.setWord(wordlist.preserveCase(listwt.front().word(), ret.second.first.word()));
	  if (wordlist.allUpper(listwt.front().word())) {
	       ret.second.second.setWord(wordlist.preserveCase(listwt.front().word(), ret.second.second.word()));
	  }
	  ret.second.first.addDesc(listwt.front().desc());
	  ret.second.second.addDesc(listwt.front().desc());
	  ret.second.first.addDesc(reportSplitCompound);
	  ret.second.second.addDesc(reportSplitCompound);
	  errorwriter.insertSplitCompound(ret.second.first.word().length(),
					 listwt.front(), ret.second.first, 
					 ret.second.second);
	  std::list<WordTag> newtags;
	  newtags.push_back(ret.second.first);
	  newtags.push_back(ret.second.second);
	  listwt.pop_front();

	  /// new
	  for (std::list<WordTag>::iterator iter = listwt.begin();
	       iter != listwt.end(); ++iter) {
	       iter->setRow(iter->row() + 1);
	  }
	  inputreader.changeReadOffset(1);

	  listwt.insert(listwt.begin(), newtags.begin(), newtags.end());
	  return true;
     }
     return false;
}

bool SplitCompound::tagAllowed(int tagid) const {
     boost::regex exp;
     const std::string& tag = wordlist.tag(tagid);
     for (std::set<std::string>::const_iterator iter = allowedClasses.begin();
	  iter != allowedClasses.end(); ++iter) {
	  exp.set_expression(*iter, boost::regbase::normal);
	  if (regex_match(tag, exp)) {
	       return true;
	  }
     }
     return false;
}

